package top.daxian1218.dust.license;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.context.ConfigurableApplicationContext;

import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author daxian
 */
public class DustLicenseApplicationListener implements ApplicationListener<ApplicationReadyEvent> {
    Logger logger = LoggerFactory.getLogger(DustLicenseApplicationListener.class);

    private long lastTimestamp = 0;
    private ConfigurableApplicationContext context;
    private ILicenseVerifier licenseVerifier;
    private final int VERIFY_INTERVAL  = 1000 * 60 * 60;


    @Override
    public void onApplicationEvent(ApplicationReadyEvent event) {
        context =  event.getApplicationContext();
        Executors.newSingleThreadExecutor().submit(() -> {
            while (true) {
                try {

                    licenseVerifier = context.getBean(ILicenseVerifier.class);
                    LicenseService licenseService = new LicenseService(licenseVerifier);
                    if (StringUtils.isEmpty(LicenseConfig.INSTANCE.getPublicKey())) {
                        licenseService.readSignInfo();
                    }

                    licenseService.generateTorrentFile();
                    if ((System.currentTimeMillis() - lastTimestamp) > VERIFY_INTERVAL) {
                        lastTimestamp = System.currentTimeMillis();
                        licenseService.verifyLicense();
                    }

                } catch (BeansException e) {
                    logger.error("Licence校验器不存在或者程序被修改, 请联系供应商", e);
                    context.close();
                    return;
                } catch (LicenseException ex) {
                    logger.error(ex.getMessage(), ex);
                    context.close();
                    return;
                }
                try {
                    Thread.sleep(1000 * 60);
                } catch (InterruptedException e) {
                    logger.error("License认证等待出错", e);
                }
            }
        });
    }


}
